@@ -1,22 +1,22 @@
-use crate::esruntime::EsRuntime;
+use crate::facades::QuickJsRuntimeFacade;
 use crate::quickjs_utils;
 use crate::quickjs_utils::{functions, get_global_q, objects, parse_args};
-use crate::quickjsruntime::QuickJsRuntime;
+use crate::quickjsruntimeadapter::QuickJsRuntimeAdapter;
 use hirofa_utils::js_utils::JsError;
 use libquickjs_sys as q;
 
 /// provides the setImmediate methods for the runtime
 /// # Example
 /// ```rust
-/// use quickjs_runtime::esruntimebuilder::EsRuntimeBuilder;
+/// use quickjs_runtime::builder::QuickJsRuntimeBuilder;
 /// use hirofa_utils::js_utils::Script;
 /// use std::time::Duration;
-/// let rt = EsRuntimeBuilder::new().build();
+/// let rt = QuickJsRuntimeBuilder::new().build();
 /// rt.eval(Script::new("test_immediate.es", "setImmediate(() => {console.log('immediate logging')});"));
 /// std::thread::sleep(Duration::from_secs(1));
 /// ```
 
-pub fn init(q_js_rt: &QuickJsRuntime) -> Result<(), JsError> {
+pub fn init(q_js_rt: &QuickJsRuntimeAdapter) -> Result<(), JsError> {
     log::trace!("setimmediate::init");
 
     q_js_rt.add_context_init_hook(|_q_js_rt, q_ctx| {
@@ -41,7 +41,7 @@ unsafe extern "C" fn set_immediate(
 
     let mut args = parse_args(context, argc, argv);
 
-    QuickJsRuntime::do_with(move |q_js_rt| {
+    QuickJsRuntimeAdapter::do_with(move |q_js_rt| {
         let q_ctx = q_js_rt.get_quickjs_context(context);
         if args.is_empty() {
             return q_ctx.report_ex("setImmediate requires at least one argument");
@@ -50,7 +50,7 @@ unsafe extern "C" fn set_immediate(
             return q_ctx.report_ex("setImmediate requires a functions as first arg");
         }
 
-        EsRuntime::add_local_task_to_event_loop(move |_q_js_rt| {
+        QuickJsRuntimeFacade::add_local_task_to_event_loop(move |_q_js_rt| {
             let func = args.remove(0);
 
             match functions::call_function(context, &func, args, None) {
