@@ -1,7 +1,7 @@
 //! Utils to compile script to bytecode and run script from bytecode
 
-use crate::quickjscontext::QuickJsContext;
-use crate::quickjsruntime::make_cstring;
+use crate::quickjsrealmadapter::QuickJsRealmAdapter;
+use crate::quickjsruntimeadapter::make_cstring;
 use crate::valueref::JSValueRef;
 use hirofa_utils::js_utils::JsError;
 use hirofa_utils::js_utils::Script;
@@ -12,11 +12,11 @@ use std::os::raw::c_void;
 ///  It can be executed with run_compiled_function().
 /// # Example
 /// ```rust
-/// use quickjs_runtime::esruntimebuilder::EsRuntimeBuilder;
+/// use quickjs_runtime::builder::QuickJsRuntimeBuilder;
 /// use hirofa_utils::js_utils::Script;
 /// use quickjs_runtime::quickjs_utils::primitives;
 /// use quickjs_runtime::quickjs_utils::compile::{compile, run_compiled_function};
-/// let rt = EsRuntimeBuilder::new().build();
+/// let rt = QuickJsRuntimeBuilder::new().build();
 /// rt.exe_rt_task_in_event_loop(|q_js_rt| {
 ///     unsafe {
 ///         let q_ctx = q_js_rt.get_main_context();
@@ -57,7 +57,7 @@ pub unsafe fn compile(context: *mut q::JSContext, script: Script) -> Result<JSVa
         format!("eval result of {}", script.get_path()).as_str(),
     );
     if ret.is_exception() {
-        let ex_opt = QuickJsContext::get_exception(context);
+        let ex_opt = QuickJsRealmAdapter::get_exception(context);
         if let Some(ex) = ex_opt {
             Err(ex)
         } else {
@@ -81,7 +81,7 @@ pub unsafe fn run_compiled_function(
     let val = q::JS_EvalFunction(context, compiled_func.clone_value_incr_rc());
     let val_ref = JSValueRef::new(context, val, false, true, "run_compiled_function result");
     if val_ref.is_exception() {
-        let ex_opt = QuickJsContext::get_exception(context);
+        let ex_opt = QuickJsRealmAdapter::get_exception(context);
         if let Some(ex) = ex_opt {
             Err(ex)
         } else {
@@ -97,11 +97,11 @@ pub unsafe fn run_compiled_function(
 /// write a function to bytecode
 /// # Example
 /// ```rust
-/// use quickjs_runtime::esruntimebuilder::EsRuntimeBuilder;
+/// use quickjs_runtime::builder::QuickJsRuntimeBuilder;
 /// use hirofa_utils::js_utils::Script;
 /// use quickjs_runtime::quickjs_utils::primitives;
 /// use quickjs_runtime::quickjs_utils::compile::{compile, run_compiled_function, to_bytecode, from_bytecode};
-/// let rt = EsRuntimeBuilder::new().build();
+/// let rt = QuickJsRuntimeBuilder::new().build();
 /// rt.exe_rt_task_in_event_loop(|q_js_rt| {
 ///     unsafe {
 ///     let q_ctx = q_js_rt.get_main_context();
@@ -110,7 +110,7 @@ pub unsafe fn run_compiled_function(
 ///     let bytecode: Vec<u8> = to_bytecode(q_ctx.context, &func);
 ///     drop(func);
 ///     assert!(!bytecode.is_empty());
-///         let func2_res = from_bytecode(q_ctx.context, bytecode);
+///         let func2_res = from_bytecode(q_ctx.context, &bytecode);
 ///         let func2 = func2_res.ok().expect("could not read bytecode");
 ///         let run_res = run_compiled_function(q_ctx.context, &func2);
 ///         let res = run_res.ok().expect("run_compiled_function failed");
@@ -123,7 +123,7 @@ pub unsafe fn run_compiled_function(
 /// # Safety
 /// When passing a context pointer please make sure the corresponding QuickJsContext is still valid
 pub unsafe fn to_bytecode(context: *mut q::JSContext, compiled_func: &JSValueRef) -> Vec<u8> {
-    assert!(compiled_func.is_compiled_function());
+    assert!(compiled_func.is_compiled_function() || compiled_func.is_module());
 
     let mut len = 0;
 
@@ -146,7 +146,7 @@ pub unsafe fn to_bytecode(context: *mut q::JSContext, compiled_func: &JSValueRef
 /// When passing a context pointer please make sure the corresponding QuickJsContext is still valid
 pub unsafe fn from_bytecode(
     context: *mut q::JSContext,
-    bytecode: Vec<u8>,
+    bytecode: &[u8],
 ) -> Result<JSValueRef, JsError> {
     assert!(!bytecode.is_empty());
     {
@@ -157,7 +157,7 @@ pub unsafe fn from_bytecode(
 
         let func_ref = JSValueRef::new(context, raw, false, true, "from_bytecode result");
         if func_ref.is_exception() {
-            let ex_opt = QuickJsContext::get_exception(context);
+            let ex_opt = QuickJsRealmAdapter::get_exception(context);
             if let Some(ex) = ex_opt {
                 Err(ex)
             } else {
@@ -173,19 +173,27 @@ pub unsafe fn from_bytecode(
 
 #[cfg(test)]
 pub mod tests {
-    use crate::esruntime::tests::init_test_rt;
-    use crate::esruntime::EsRuntime;
-    use crate::esruntimebuilder::EsRuntimeBuilder;
+    use crate::builder::QuickJsRuntimeBuilder;
+    use crate::facades::tests::init_test_rt;
     use crate::quickjs_utils::compile::{
         compile, from_bytecode, run_compiled_function, to_bytecode,
     };
+    use crate::quickjs_utils::modules::compile_module;
     use crate::quickjs_utils::primitives;
+    use crate::quickjsrealmadapter::QuickJsRealmAdapter;
+    use backtrace::Backtrace;
+    use futures::executor::block_on;
+    use hirofa_utils::js_utils::facades::values::JsValueFacade;
+    use hirofa_utils::js_utils::facades::{JsRuntimeBuilder, JsRuntimeFacade};
+    use hirofa_utils::js_utils::modules::CompiledModuleLoader;
     use hirofa_utils::js_utils::Script;
+    use log::LevelFilter;
+    use std::panic;
     use std::sync::Arc;
 
     #[test]
     fn test_compile() {
-        let rt: Arc<EsRuntime> = init_test_rt();
+        let rt = init_test_rt();
 
         rt.exe_rt_task_in_event_loop(|q_js_rt| {
             let q_ctx = q_js_rt.get_main_context();
@@ -202,7 +210,7 @@ pub mod tests {
             let bytecode: Vec<u8> = unsafe { to_bytecode(q_ctx.context, &func) };
             drop(func);
             assert!(!bytecode.is_empty());
-            let func2_res = unsafe { from_bytecode(q_ctx.context, bytecode) };
+            let func2_res = unsafe { from_bytecode(q_ctx.context, &bytecode) };
             let func2 = func2_res.ok().expect("could not read bytecode");
             let run_res = unsafe { run_compiled_function(q_ctx.context, &func2) };
             match run_res {
@@ -220,7 +228,7 @@ pub mod tests {
 
     #[test]
     fn test_bytecode() {
-        let rt: Arc<EsRuntime> = init_test_rt();
+        let rt = init_test_rt();
         rt.exe_rt_task_in_event_loop(|q_js_rt| unsafe {
             let q_ctx = q_js_rt.get_main_context();
             let func_res = compile(
@@ -234,7 +242,7 @@ pub mod tests {
             let bytecode: Vec<u8> = to_bytecode(q_ctx.context, &func);
             drop(func);
             assert!(!bytecode.is_empty());
-            let func2_res = from_bytecode(q_ctx.context, bytecode);
+            let func2_res = from_bytecode(q_ctx.context, &bytecode);
             let func2 = func2_res.ok().expect("could not read bytecode");
             let run_res = run_compiled_function(q_ctx.context, &func2);
 
@@ -253,7 +261,7 @@ pub mod tests {
 
     #[test]
     fn test_bytecode_bad_compile() {
-        let rt = EsRuntimeBuilder::new().build();
+        let rt = QuickJsRuntimeBuilder::new().build();
         rt.exe_rt_task_in_event_loop(|q_js_rt| {
             let q_ctx = q_js_rt.get_main_context();
 
@@ -272,7 +280,7 @@ pub mod tests {
 
     #[test]
     fn test_bytecode_bad_run() {
-        let rt = EsRuntimeBuilder::new().build();
+        let rt = QuickJsRuntimeBuilder::new().build();
         rt.exe_rt_task_in_event_loop(|q_js_rt| unsafe {
             let q_ctx = q_js_rt.get_main_context();
 
@@ -291,7 +299,7 @@ pub mod tests {
 
             assert!(!bytecode.is_empty());
 
-            let func2_res = from_bytecode(q_ctx.context, bytecode);
+            let func2_res = from_bytecode(q_ctx.context, &bytecode);
             let func2 = func2_res.ok().expect("could not read bytecode");
             //should fail the second time you run this because abcdef is already defined
 
@@ -311,4 +319,86 @@ pub mod tests {
             assert_eq!(1, func2.get_ref_count());
         });
     }
+
+    lazy_static! {
+        static ref COMPILED_BYTES: Arc<Vec<u8>> = init_bytes();
+    }
+
+    fn init_bytes() -> Arc<Vec<u8>> {
+        // in order to init our bytes fgor our module we lazy init a rt
+        let rt = QuickJsRuntimeBuilder::new().build();
+        rt.js_loop_realm_sync(None, |_rt, realm| unsafe {
+            let script = Script::new(
+                "test_module.js",
+                "export function someFunction(a, b){return a*b;};",
+            );
+
+            let module = compile_module(realm.context, script)
+                .ok()
+                .expect("compile failed");
+
+            Arc::new(to_bytecode(realm.context, &module))
+        })
+    }
+
+    struct Cml {}
+    impl CompiledModuleLoader<QuickJsRealmAdapter> for Cml {
+        fn normalize_path(
+            &self,
+            _q_ctx: &QuickJsRealmAdapter,
+            _ref_path: &str,
+            path: &str,
+        ) -> Option<String> {
+            Some(path.to_string())
+        }
+
+        fn load_module(&self, _q_ctx: &QuickJsRealmAdapter, _absolute_path: &str) -> Arc<Vec<u8>> {
+            COMPILED_BYTES.clone()
+        }
+    }
+
+    #[test]
+    fn test_bytecode_module() {
+        panic::set_hook(Box::new(|panic_info| {
+            let backtrace = Backtrace::new();
+            println!(
+                "thread panic occurred: {}\nbacktrace: {:?}",
+                panic_info, backtrace
+            );
+            log::error!(
+                "thread panic occurred: {}\nbacktrace: {:?}",
+                panic_info,
+                backtrace
+            );
+        }));
+
+        simple_logging::log_to_file("quickjs_runtime.log", LevelFilter::max())
+            .ok()
+            .expect("could not init logger");
+
+        let rt = QuickJsRuntimeBuilder::new()
+            .js_compiled_module_loader(Cml {})
+            .build();
+
+        let test_script = Script::new(
+            "test_bytecode_module.js",
+            "import('testcompiledmodule').then((mod) => {return mod.someFunction(3, 5);})",
+        );
+        let res_fut = rt.js_eval(None, test_script);
+        let res_prom = block_on(res_fut).ok().expect("script failed");
+        let rti_weak = rt.js_get_runtime_facade_inner();
+        if let JsValueFacade::JsPromise { cached_promise } = res_prom {
+            let rti = rti_weak.upgrade().expect("invalid state");
+            let prom_res_fut = cached_promise.js_get_promise_result(&*rti);
+            let prom_res = block_on(prom_res_fut)
+                .ok()
+                .expect("prom failed")
+                .ok()
+                .expect("prom was rejected");
+            assert!(prom_res.is_i32());
+            assert_eq!(prom_res.get_i32(), 15);
+        } else {
+            panic!("did not get a prom");
+        }
+    }
 }
