@@ -1,7 +1,7 @@
 //! utils for getting and reporting exceptions
 
 use crate::quickjs_utils::{objects, primitives};
-use crate::quickjscontext::QuickJsContext;
+use crate::quickjsrealmadapter::QuickJsRealmAdapter;
 use crate::valueref::{JSValueRef, TAG_EXCEPTION};
 use hirofa_utils::js_utils::JsError;
 use libquickjs_sys as q;
@@ -43,11 +43,27 @@ pub unsafe fn error_to_js_error(context: *mut q::JSContext, exception_ref: &JSVa
     let stack_ref = objects::get_property(context, exception_ref, "stack")
         .ok()
         .unwrap();
-    let stack_string;
+    let mut stack_string = "".to_string();
+
+    let stack2_ref = objects::get_property(context, exception_ref, "stack2")
+        .ok()
+        .unwrap();
+    if stack2_ref.is_string() {
+        stack_string.push_str(
+            primitives::to_string(context, &stack2_ref)
+                .ok()
+                .unwrap()
+                .as_str(),
+        );
+    }
+
     if stack_ref.is_string() {
-        stack_string = primitives::to_string(context, &stack_ref).ok().unwrap();
-    } else {
-        stack_string = "".to_string();
+        stack_string.push_str(
+            primitives::to_string(context, &stack_ref)
+                .ok()
+                .unwrap()
+                .as_str(),
+        );
     }
 
     JsError::new(name_string, message_string, stack_string)
@@ -85,14 +101,14 @@ pub unsafe fn new_error(
     objects::set_property(
         context,
         &obj_ref,
-        "stack",
+        "stack2",
         &primitives::from_string(context, stack)?,
     )?;
     Ok(obj_ref)
 }
 
 /// See if a JSValueRef is an Error object
-pub fn is_error_q(q_ctx: &QuickJsContext, obj_ref: &JSValueRef) -> bool {
+pub fn is_error_q(q_ctx: &QuickJsRealmAdapter, obj_ref: &JSValueRef) -> bool {
     unsafe { is_error(q_ctx.context, obj_ref) }
 }
 
@@ -122,19 +138,45 @@ pub unsafe fn throw(context: *mut q::JSContext, error: JSValueRef) -> q::JSValue
 
 #[cfg(test)]
 pub mod tests {
-    use crate::esruntime::tests::init_test_rt;
-    use crate::esruntime::EsRuntime;
     use crate::esvalue::EsValueConvertible;
+    use crate::facades::tests::init_test_rt;
     use crate::quickjs_utils::functions;
     use hirofa_utils::js_utils::Script;
-    use std::sync::Arc;
     use std::time::Duration;
 
     #[test]
-    fn test_ex() {
+    fn test_ex_nat() {
         // check if stacktrace is preserved when invoking native methods
 
-        let rt: Arc<EsRuntime> = init_test_rt();
+        let rt = init_test_rt();
+        let res = rt.eval_sync(Script::new(
+            "ex.js",
+            "console.log('foo');\nconsole.log('bar');let a = __c_v__ * 7;",
+        ));
+        let ex = res.err().expect("sciprt should have failed;");
+
+        assert_eq!(ex.get_message(), "'__c_v__' is not defined");
+    }
+
+    #[test]
+    fn test_ex0() {
+        // check if stacktrace is preserved when invoking native methods
+
+        let rt = init_test_rt();
+        let res = rt.eval_sync(Script::new(
+            "ex.js",
+            "console.log('foo');\nconsole.log('bar');let a = __c_v__ * 7;",
+        ));
+        let ex = res.err().expect("sciprt should have failed;");
+
+        assert_eq!(ex.get_message(), "'__c_v__' is not defined");
+    }
+
+    #[test]
+    fn test_ex1() {
+        // check if stacktrace is preserved when invoking native methods
+
+        let rt = init_test_rt();
         rt.set_function(vec![], "test_consume", |_q_ctx, args| {
             // args[0] is a function i'll want to call
             let func_esvf = &args[0];
