@@ -1,8 +1,8 @@
 //! utils to create and invoke functions
 
 use crate::quickjs_utils::{atoms, errors, objects, parse_args, primitives};
-use crate::quickjscontext::QuickJsContext;
-use crate::quickjsruntime::{make_cstring, QuickJsRuntime};
+use crate::quickjsrealmadapter::QuickJsRealmAdapter;
+use crate::quickjsruntimeadapter::{make_cstring, QuickJsRuntimeAdapter};
 use crate::valueref::JSValueRef;
 use hirofa_utils::auto_id_map::AutoIdMap;
 use hirofa_utils::js_utils::JsError;
@@ -59,21 +59,32 @@ pub unsafe fn parse_function(
 
     let file_name = format!("compile_func_{}.es", name);
 
-    let ret = QuickJsContext::eval_ctx(context, Script::new(&file_name, &src))?;
+    let ret = QuickJsRealmAdapter::eval_ctx(context, Script::new(&file_name, &src))?;
 
     debug_assert!(is_function(context, &ret));
 
     Ok(ret)
 }
 
+/// call a function
+pub fn call_function_q_ref_args(
+    q_ctx: &QuickJsRealmAdapter,
+    function_ref: &JSValueRef,
+    arguments: &[&JSValueRef],
+    this_ref_opt: Option<&JSValueRef>,
+) -> Result<JSValueRef, JsError> {
+    unsafe { call_function_ref_args(q_ctx.context, function_ref, arguments, this_ref_opt) }
+}
+
 /// call a function
 pub fn call_function_q(
-    q_ctx: &QuickJsContext,
+    q_ctx: &QuickJsRealmAdapter,
     function_ref: &JSValueRef,
     arguments: Vec<JSValueRef>,
     this_ref_opt: Option<&JSValueRef>,
 ) -> Result<JSValueRef, JsError> {
-    unsafe { call_function(q_ctx.context, function_ref, arguments, this_ref_opt) }
+    let r: Vec<&JSValueRef> = arguments.iter().collect();
+    unsafe { call_function_ref_args(q_ctx.context, function_ref, &r, this_ref_opt) }
 }
 
 /// call a function
@@ -84,6 +95,19 @@ pub unsafe fn call_function(
     function_ref: &JSValueRef,
     arguments: Vec<JSValueRef>,
     this_ref_opt: Option<&JSValueRef>,
+) -> Result<JSValueRef, JsError> {
+    let r: Vec<&JSValueRef> = arguments.iter().collect();
+    call_function_ref_args(context, function_ref, &r, this_ref_opt)
+}
+
+/// call a function
+/// # Safety
+/// When passing a context pointer please make sure the corresponding QuickJsContext is still valid
+pub unsafe fn call_function_ref_args(
+    context: *mut q::JSContext,
+    function_ref: &JSValueRef,
+    arguments: &[&JSValueRef],
+    this_ref_opt: Option<&JSValueRef>,
 ) -> Result<JSValueRef, JsError> {
     log::trace!("functions::call_function()");
 
@@ -114,7 +138,7 @@ pub unsafe fn call_function(
     let res_ref = JSValueRef::new(context, res, false, true, "call_function result");
 
     if res_ref.is_exception() {
-        if let Some(ex) = QuickJsContext::get_exception(context) {
+        if let Some(ex) = QuickJsRealmAdapter::get_exception(context) {
             Err(ex)
         } else {
             Err(JsError::new_str(
@@ -137,7 +161,7 @@ pub fn JS_Invoke(
  */
 
 pub fn invoke_member_function_q(
-    q_ctx: &QuickJsContext,
+    q_ctx: &QuickJsRealmAdapter,
     obj_ref: &JSValueRef,
     function_name: &str,
     arguments: Vec<JSValueRef>,
@@ -184,7 +208,7 @@ pub unsafe fn invoke_member_function(
     );
 
     if res_ref.is_exception() {
-        if let Some(ex) = QuickJsContext::get_exception(context) {
+        if let Some(ex) = QuickJsRealmAdapter::get_exception(context) {
             Err(ex)
         } else {
             Err(JsError::new_str(
@@ -197,7 +221,10 @@ pub unsafe fn invoke_member_function(
 }
 
 /// call an objects to_String method or convert a value to string
-pub fn call_to_string_q(q_ctx: &QuickJsContext, obj_ref: &JSValueRef) -> Result<String, JsError> {
+pub fn call_to_string_q(
+    q_ctx: &QuickJsRealmAdapter,
+    obj_ref: &JSValueRef,
+) -> Result<String, JsError> {
     unsafe { call_to_string(q_ctx.context, obj_ref) }
 }
 
@@ -241,7 +268,7 @@ pub unsafe fn call_to_string(
 }
 
 /// see if an Object is an instance of Function
-pub fn is_function_q(q_ctx: &QuickJsContext, obj_ref: &JSValueRef) -> bool {
+pub fn is_function_q(q_ctx: &QuickJsRealmAdapter, obj_ref: &JSValueRef) -> bool {
     unsafe { is_function(q_ctx.context, obj_ref) }
 }
 
@@ -259,7 +286,7 @@ pub unsafe fn is_function(context: *mut q::JSContext, obj_ref: &JSValueRef) -> b
 }
 
 /// see if an Object is an instance of Function and is a constructor (can be instantiated with new keyword)
-pub fn is_constructor_q(q_ctx: &QuickJsContext, obj_ref: &JSValueRef) -> bool {
+pub fn is_constructor_q(q_ctx: &QuickJsRealmAdapter, obj_ref: &JSValueRef) -> bool {
     unsafe { is_constructor(q_ctx.context, obj_ref) }
 }
 
@@ -277,7 +304,7 @@ pub unsafe fn is_constructor(context: *mut q::JSContext, obj_ref: &JSValueRef) -
 
 /// call a constructor (instantiate an Object)
 pub fn call_constructor_q(
-    q_ctx: &QuickJsContext,
+    q_ctx: &QuickJsRealmAdapter,
     constructor_ref: &JSValueRef,
     arguments: &[JSValueRef],
 ) -> Result<JSValueRef, JsError> {
@@ -323,7 +350,7 @@ pub unsafe fn call_constructor(
     );
 
     if res_ref.is_exception() {
-        if let Some(ex) = QuickJsContext::get_exception(context) {
+        if let Some(ex) = QuickJsRealmAdapter::get_exception(context) {
             Err(ex)
         } else {
             Err(JsError::new_str(
@@ -337,7 +364,7 @@ pub unsafe fn call_constructor(
 
 /// create a new Function object which calls a native method
 pub fn new_native_function_q(
-    q_ctx: &QuickJsContext,
+    q_ctx: &QuickJsRealmAdapter,
     name: &str,
     func: q::JSCFunction,
     arg_count: i32,
@@ -401,7 +428,7 @@ pub unsafe fn new_native_function(
 
 /// create a new Function object which calls a native method (with data)
 pub fn new_native_function_data_q(
-    q_ctx: &QuickJsContext,
+    q_ctx: &QuickJsRealmAdapter,
     func: q::JSCFunctionData,
     name: &str,
     arg_count: i32,
@@ -487,7 +514,7 @@ thread_local! {
 
         CALLBACK_CLASS_DEF.with(|cd_rc| {
             let class_def = &*cd_rc.borrow();
-            QuickJsRuntime::do_with(|q_js_rt| {
+            QuickJsRuntimeAdapter::do_with(|q_js_rt| {
                 let res = unsafe { q::JS_NewClass(q_js_rt.runtime, class_id, class_def) };
                 log::trace!("callback: new class res {}", res);
                 // todo res should be 0 for ok
@@ -497,22 +524,29 @@ thread_local! {
         RefCell::new(class_id)
     };
 
-    static CALLBACK_REGISTRY: RefCell<AutoIdMap<Box<Callback>>> = {
+    static CALLBACK_REGISTRY: RefCell<AutoIdMap<(String, Box<Callback>)>> = {
         RefCell::new(AutoIdMap::new_with_max_size(i32::MAX as usize))
     };
 
     static CALLBACK_IDS: RefCell<HashSet<Box<i32>>> = RefCell::new(HashSet::new());
 }
+
+pub(crate) fn init_statics() {
+    CALLBACK_CLASS_ID.with(|_rc| {
+        //
+    });
+}
+
 /// create a new Function which is backed by a closure
 /// # Example
 /// ```rust
-/// use quickjs_runtime::esruntimebuilder::EsRuntimeBuilder;
+/// use quickjs_runtime::builder::QuickJsRuntimeBuilder;
 /// use quickjs_runtime::quickjs_utils::functions::new_function_q;
 /// use quickjs_runtime::quickjs_utils::primitives::from_i32;
 /// use quickjs_runtime::quickjs_utils::get_global_q;
 /// use quickjs_runtime::quickjs_utils::objects::set_property_q;
 /// use hirofa_utils::js_utils::Script;
-/// let rt = EsRuntimeBuilder::new().build();
+/// let rt = QuickJsRuntimeBuilder::new().build();
 /// rt.exe_rt_task_in_event_loop(|q_js_rt| {
 ///     let q_ctx = q_js_rt.get_main_context();
 ///     // create a function which always returns 1253
@@ -524,17 +558,18 @@ thread_local! {
 /// rt.eval_sync(Script::new("new_function_q.es", "let a = myFunc7654(); if (a !== 1253) {throw Error('a was not 1253')}")).ok().expect("script failed");
 /// ```
 pub fn new_function_q<F>(
-    q_ctx: &QuickJsContext,
+    q_ctx: &QuickJsRealmAdapter,
     name: &str,
     func: F,
     arg_count: u32,
 ) -> Result<JSValueRef, JsError>
 where
-    F: Fn(&QuickJsContext, &JSValueRef, &[JSValueRef]) -> Result<JSValueRef, JsError> + 'static,
+    F: Fn(&QuickJsRealmAdapter, &JSValueRef, &[JSValueRef]) -> Result<JSValueRef, JsError>
+        + 'static,
 {
     let func_raw = move |ctx: *mut q::JSContext, this: &JSValueRef, args: &[JSValueRef]| {
         log::trace!("new_function_q outer");
-        QuickJsRuntime::do_with(|q_js_rt| {
+        QuickJsRuntimeAdapter::do_with(|q_js_rt| {
             log::trace!("new_function_q inner");
             func(unsafe { q_js_rt.get_quickjs_context(ctx) }, this, args)
         })
@@ -563,7 +598,7 @@ where
 
     let callback_id = CALLBACK_REGISTRY.with(|registry_rc| {
         let registry = &mut *registry_rc.borrow_mut();
-        registry.insert(Box::new(func))
+        registry.insert((name.to_string(), Box::new(func)))
     });
     log::trace!("new_function callback_id = {}", callback_id);
 
@@ -589,7 +624,7 @@ where
     );
 
     if class_val_ref.is_exception() {
-        return if let Some(e) = QuickJsContext::get_exception(context) {
+        return if let Some(e) = QuickJsRealmAdapter::get_exception(context) {
             Err(e)
         } else {
             Err(JsError::new_str("could not create callback class"))
@@ -617,12 +652,13 @@ where
 
 #[cfg(test)]
 pub mod tests {
-    use crate::esruntime::tests::init_test_rt;
+    use crate::facades::tests::init_test_rt;
     use crate::quickjs_utils::functions::{
         call_function_q, call_to_string_q, invoke_member_function_q, new_function_q,
     };
     use crate::quickjs_utils::{functions, objects, primitives};
-    use hirofa_utils::js_utils::Script;
+    use hirofa_utils::js_utils::adapters::JsRealmAdapter;
+    use hirofa_utils::js_utils::{JsError, Script};
     use std::time::Duration;
 
     #[test]
@@ -852,6 +888,41 @@ pub mod tests {
         });
         std::thread::sleep(Duration::from_secs(1));
     }
+
+    #[test]
+    fn test_ex() {
+        let rt = init_test_rt();
+
+        let err = rt.exe_rt_task_in_event_loop(|q_js_rt| {
+            let q_ctx = q_js_rt.get_main_context();
+
+            q_ctx
+                .js_install_function(
+                    &["test_927"],
+                    "testMe",
+                    |_rt, _q_ctx, _this_ref, _args| {
+                        log::trace!("native callback invoked");
+                        Err(JsError::new_str("poof"))
+                    },
+                    0,
+                )
+                .ok()
+                .expect("could not install func");
+
+            let err = q_ctx
+                .eval(Script::new(
+                    "test_927.es",
+                    "console.log('foo');test_927.testMe();",
+                ))
+                .err()
+                .expect("did not get err");
+
+            format!("{}", err)
+        });
+
+        assert!(err.contains("[testMe]"));
+        assert!(err.contains("test_927.es"));
+    }
 }
 
 unsafe extern "C" fn callback_finalizer(_rt: *mut q::JSRuntime, val: q::JSValue) {
@@ -863,11 +934,11 @@ unsafe extern "C" fn callback_finalizer(_rt: *mut q::JSRuntime, val: q::JSValue)
 
     trace!("callback_finalizer called, id={}", callback_id);
 
-    CALLBACK_IDS.with(|rc| {
+    let _ = CALLBACK_IDS.try_with(|rc| {
         let ids = &mut *rc.borrow_mut();
         ids.remove(&callback_id);
     });
-    CALLBACK_REGISTRY.with(|rc| {
+    let _ = CALLBACK_REGISTRY.try_with(|rc| {
         let registry = &mut *rc.borrow_mut();
 
         let rid = callback_id as usize;
@@ -897,7 +968,7 @@ unsafe extern "C" fn callback_function(
 
     CALLBACK_REGISTRY.with(|registry_rc| {
         let registry = &*registry_rc.borrow();
-        if let Some(callback) = registry.get(&(callback_id as usize)) {
+        if let Some((name, callback)) = registry.get(&(callback_id as usize)) {
             let args_vec = parse_args(ctx, argc, argv);
 
             let this_ref = JSValueRef::new(ctx, this_val, true, true, "callback_function this_val");
@@ -908,11 +979,11 @@ unsafe extern "C" fn callback_function(
             match callback_res {
                 Ok(res) => res.clone_value_incr_rc(),
                 Err(e) => {
-                    let message =
-                        format!("\n{} at\nnative_code\n{}", e.get_message(), e.get_stack());
-                    let err = errors::new_error(ctx, e.get_name(), message.as_str(), e.get_stack())
-                        .ok()
-                        .expect("could not create err");
+                    let nat_stack = format!("   at native_function [{}]\n{}", name, e.get_stack());
+                    let err =
+                        errors::new_error(ctx, e.get_name(), e.get_message(), nat_stack.as_str())
+                            .ok()
+                            .expect("could not create err");
                     errors::throw(ctx, err)
                 }
             }
@@ -924,13 +995,13 @@ unsafe extern "C" fn callback_function(
 
 #[cfg(test)]
 pub mod tests2 {
-    use crate::esruntimebuilder::EsRuntimeBuilder;
+    use crate::builder::QuickJsRuntimeBuilder;
     use crate::quickjs_utils::functions::{new_function_q, CALLBACK_IDS, CALLBACK_REGISTRY};
     use crate::quickjs_utils::new_null_ref;
 
     #[test]
     fn test_function() {
-        let rt = EsRuntimeBuilder::new().build();
+        let rt = QuickJsRuntimeBuilder::new().build();
         rt.exe_rt_task_in_event_loop(|q_js_rt| {
             let q_ctx = q_js_rt.get_main_context();
             let func = new_function_q(
