@@ -1,14 +1,14 @@
 use crate::quickjs_utils;
 use crate::quickjs_utils::functions;
 use crate::quickjs_utils::objects::is_instance_of_by_name;
-use crate::quickjscontext::QuickJsContext;
-use crate::quickjsruntime::QuickJsRuntime;
+use crate::quickjsrealmadapter::QuickJsRealmAdapter;
+use crate::quickjsruntimeadapter::QuickJsRuntimeAdapter;
 use crate::valueref::JSValueRef;
-use hirofa_utils::js_utils::adapters::{JsPromiseAdapter, JsRealmAdapter};
+use hirofa_utils::js_utils::adapters::JsPromiseAdapter;
 use hirofa_utils::js_utils::JsError;
 use libquickjs_sys as q;
 
-pub fn is_promise_q(context: &QuickJsContext, obj_ref: &JSValueRef) -> bool {
+pub fn is_promise_q(context: &QuickJsRealmAdapter, obj_ref: &JSValueRef) -> bool {
     unsafe { is_promise(context.context, obj_ref) }
 }
 
@@ -32,7 +32,7 @@ impl PromiseRef {
         self.promise_obj_ref.clone()
     }
 
-    pub fn resolve_q(&self, q_ctx: &QuickJsContext, value: JSValueRef) -> Result<(), JsError> {
+    pub fn resolve_q(&self, q_ctx: &QuickJsRealmAdapter, value: JSValueRef) -> Result<(), JsError> {
         unsafe { self.resolve(q_ctx.context, value) }
     }
     /// # Safety
@@ -51,7 +51,7 @@ impl PromiseRef {
         )?;
         Ok(())
     }
-    pub fn reject_q(&self, q_ctx: &QuickJsContext, value: JSValueRef) -> Result<(), JsError> {
+    pub fn reject_q(&self, q_ctx: &QuickJsRealmAdapter, value: JSValueRef) -> Result<(), JsError> {
         unsafe { self.reject(q_ctx.context, value) }
     }
     /// # Safety
@@ -82,12 +82,10 @@ impl Clone for PromiseRef {
     }
 }
 
-impl JsPromiseAdapter for PromiseRef {
-    type JsRuntimeAdapterType = QuickJsRuntime;
-
+impl JsPromiseAdapter<QuickJsRealmAdapter> for PromiseRef {
     fn js_promise_resolve(
         &self,
-        context: &QuickJsContext,
+        context: &QuickJsRealmAdapter,
         resolution: &JSValueRef,
     ) -> Result<(), JsError> {
         self.resolve_q(context, resolution.clone())
@@ -95,77 +93,18 @@ impl JsPromiseAdapter for PromiseRef {
 
     fn js_promise_reject(
         &self,
-        context: &QuickJsContext,
+        context: &QuickJsRealmAdapter,
         rejection: &JSValueRef,
     ) -> Result<(), JsError> {
         self.reject_q(context, rejection.clone())
     }
 
-    fn js_promise_add_reactions<F>(
-        &self,
-        context: &QuickJsContext,
-        then: Option<F>,
-        catch: Option<F>,
-        finally: Option<F>,
-    ) -> Result<(), JsError>
-    where
-        F: Fn(&JSValueRef) -> Result<(), JsError> + 'static,
-    {
-        let then = then.map(|t_clos| {
-            context
-                .js_function_create(
-                    "then",
-                    move |_ctx, _this, args| {
-                        t_clos(&args[0])?;
-                        Ok(crate::quickjs_utils::new_null_ref())
-                    },
-                    1,
-                )
-                .ok()
-                .expect("could not create function")
-        });
-        let catch = catch.map(|t_clos| {
-            context
-                .js_function_create(
-                    "catch",
-                    move |_ctx, _this, args| {
-                        t_clos(&args[0])?;
-                        Ok(crate::quickjs_utils::new_null_ref())
-                    },
-                    1,
-                )
-                .ok()
-                .expect("could not create function")
-        });
-        let finally = finally.map(|t_clos| {
-            context
-                .js_function_create(
-                    "finally",
-                    move |_ctx, _this, args| {
-                        t_clos(&args[0])?;
-                        Ok(crate::quickjs_utils::new_null_ref())
-                    },
-                    1,
-                )
-                .ok()
-                .expect("could not create function")
-        });
-
-        crate::quickjs_utils::promises::add_promise_reactions_q(
-            context,
-            &self.promise_obj_ref,
-            then,
-            catch,
-            finally,
-        )
-    }
-
-    fn js_promise_get_value(&self) -> JSValueRef {
+    fn js_promise_get_value(&self, _realm: &QuickJsRealmAdapter) -> JSValueRef {
         self.promise_obj_ref.clone()
     }
 }
 
-pub fn new_promise_q(q_ctx: &QuickJsContext) -> Result<PromiseRef, JsError> {
+pub fn new_promise_q(q_ctx: &QuickJsRealmAdapter) -> Result<PromiseRef, JsError> {
     unsafe { new_promise(q_ctx.context) }
 }
 
@@ -219,7 +158,7 @@ pub unsafe fn new_promise(context: *mut q::JSContext) -> Result<PromiseRef, JsEr
     })
 }
 
-pub(crate) fn init_promise_rejection_tracker(q_js_rt: &QuickJsRuntime) {
+pub(crate) fn init_promise_rejection_tracker(q_js_rt: &QuickJsRuntimeAdapter) {
     let tracker: q::JSHostPromiseRejectionTracker = Some(promise_rejection_tracker);
 
     unsafe {
@@ -228,7 +167,7 @@ pub(crate) fn init_promise_rejection_tracker(q_js_rt: &QuickJsRuntime) {
 }
 
 pub fn add_promise_reactions_q(
-    context: &QuickJsContext,
+    context: &QuickJsRealmAdapter,
     promise_obj_ref: &JSValueRef,
     then_func_obj_ref_opt: Option<JSValueRef>,
     catch_func_obj_ref_opt: Option<JSValueRef>,
@@ -260,7 +199,7 @@ pub unsafe fn add_promise_reactions(
     if let Some(then_func_obj_ref) = then_func_obj_ref_opt {
         functions::invoke_member_function(
             context,
-            &promise_obj_ref,
+            promise_obj_ref,
             "then",
             vec![then_func_obj_ref],
         )?;
@@ -268,7 +207,7 @@ pub unsafe fn add_promise_reactions(
     if let Some(catch_func_obj_ref) = catch_func_obj_ref_opt {
         functions::invoke_member_function(
             context,
-            &promise_obj_ref,
+            promise_obj_ref,
             "catch",
             vec![catch_func_obj_ref],
         )?;
@@ -276,7 +215,7 @@ pub unsafe fn add_promise_reactions(
     if let Some(finally_func_obj_ref) = finally_func_obj_ref_opt {
         functions::invoke_member_function(
             context,
-            &promise_obj_ref,
+            promise_obj_ref,
             "finally",
             vec![finally_func_obj_ref],
         )?;
@@ -316,11 +255,11 @@ unsafe extern "C" fn promise_rejection_tracker(
 
 #[cfg(test)]
 pub mod tests {
-    use crate::esruntime::tests::init_test_rt;
     use crate::esvalue::EsValueFacade;
+    use crate::facades::tests::init_test_rt;
     use crate::quickjs_utils::promises::{add_promise_reactions_q, is_promise_q, new_promise_q};
     use crate::quickjs_utils::{functions, new_null_ref, primitives};
-    use crate::quickjsruntime::QuickJsRuntime;
+    use crate::quickjsruntimeadapter::QuickJsRuntimeAdapter;
     use hirofa_utils::js_utils::Script;
     use std::time::Duration;
 
@@ -488,9 +427,9 @@ pub mod tests {
 
         let rt = init_test_rt();
 
-        let mut esvf_res = rt.exe_task(|| {
-            QuickJsRuntime::create_context("test").ok().expect("create ctx failed");
-            QuickJsRuntime::do_with(|q_js_rt| {
+        let mut esvf_res = rt.exe_task_in_event_loop(|| {
+            QuickJsRuntimeAdapter::create_context("test").ok().expect("create ctx failed");
+            QuickJsRuntimeAdapter::do_with(|q_js_rt| {
                 let q_ctx = q_js_rt.get_context("test");
 
                 let script = "(new Promise((resolve, reject) => {resolve({a: 7});}).then((obj) => {return {b: obj.a * 5}}));";
@@ -513,8 +452,8 @@ pub mod tests {
         let i = b.get_i32();
         assert_eq!(i, 5 * 7);
 
-        rt.exe_task(|| {
-            QuickJsRuntime::remove_context("test");
+        rt.exe_task_in_event_loop(|| {
+            QuickJsRuntimeAdapter::remove_context("test");
         })
     }
 }
