@@ -1,9 +1,11 @@
+//! utils for implementing proxy classes which can be used to use rust structs from JS (define method/getters/setters/etc)
+
 use crate::quickjs_utils;
 use crate::quickjs_utils::functions::new_native_function_q;
 use crate::quickjs_utils::primitives::from_string;
 use crate::quickjs_utils::{atoms, errors, functions, objects, parse_args, primitives};
-use crate::quickjscontext::QuickJsContext;
-use crate::quickjsruntime::QuickJsRuntime;
+use crate::quickjsrealmadapter::QuickJsRealmAdapter;
+use crate::quickjsruntimeadapter::QuickJsRuntimeAdapter;
 use crate::valueref::JSValueRef;
 use hirofa_utils::js_utils::JsError;
 use libquickjs_sys as q;
@@ -17,18 +19,21 @@ use std::rc::Rc;
 pub mod eventtarget;
 
 pub type ProxyConstructor =
-    dyn Fn(&QuickJsContext, usize, Vec<JSValueRef>) -> Result<(), JsError> + 'static;
-pub type ProxyFinalizer = dyn Fn(&QuickJsContext, usize) + 'static;
+    dyn Fn(&QuickJsRealmAdapter, usize, Vec<JSValueRef>) -> Result<(), JsError> + 'static;
+pub type ProxyFinalizer = dyn Fn(&QuickJsRealmAdapter, usize) + 'static;
 pub type ProxyMethod =
-    dyn Fn(&QuickJsContext, &usize, Vec<JSValueRef>) -> Result<JSValueRef, JsError> + 'static;
+    dyn Fn(&QuickJsRealmAdapter, &usize, Vec<JSValueRef>) -> Result<JSValueRef, JsError> + 'static;
 pub type ProxyNativeMethod = q::JSCFunction;
 pub type ProxyStaticMethod =
-    dyn Fn(&QuickJsContext, Vec<JSValueRef>) -> Result<JSValueRef, JsError> + 'static;
+    dyn Fn(&QuickJsRealmAdapter, Vec<JSValueRef>) -> Result<JSValueRef, JsError> + 'static;
 pub type ProxyStaticNativeMethod = q::JSCFunction;
-pub type ProxyStaticGetter = dyn Fn(&QuickJsContext) -> Result<JSValueRef, JsError> + 'static;
-pub type ProxyStaticSetter = dyn Fn(&QuickJsContext, JSValueRef) -> Result<(), JsError> + 'static;
-pub type ProxyGetter = dyn Fn(&QuickJsContext, &usize) -> Result<JSValueRef, JsError> + 'static;
-pub type ProxySetter = dyn Fn(&QuickJsContext, &usize, JSValueRef) -> Result<(), JsError> + 'static;
+pub type ProxyStaticGetter = dyn Fn(&QuickJsRealmAdapter) -> Result<JSValueRef, JsError> + 'static;
+pub type ProxyStaticSetter =
+    dyn Fn(&QuickJsRealmAdapter, JSValueRef) -> Result<(), JsError> + 'static;
+pub type ProxyGetter =
+    dyn Fn(&QuickJsRealmAdapter, &usize) -> Result<JSValueRef, JsError> + 'static;
+pub type ProxySetter =
+    dyn Fn(&QuickJsRealmAdapter, &usize, JSValueRef) -> Result<(), JsError> + 'static;
 
 static CNAME: &str = "ProxyInstanceClass\0";
 static SCNAME: &str = "ProxyStaticClass\0";
@@ -87,7 +92,7 @@ thread_local! {
 
         PROXY_STATIC_CLASS_DEF.with(|cd_rc| {
             let class_def = &*cd_rc.borrow();
-            QuickJsRuntime::do_with(|q_js_rt| {
+            QuickJsRuntimeAdapter::do_with(|q_js_rt| {
                 let res = unsafe { q::JS_NewClass(q_js_rt.runtime, class_id, class_def) };
                 log::trace!("new static class res {}", res);
                 // todo res should be 0 for ok
@@ -103,7 +108,7 @@ thread_local! {
 
         PROXY_INSTANCE_CLASS_DEF.with(|cd_rc| {
             let class_def = &*cd_rc.borrow();
-            QuickJsRuntime::do_with(|q_js_rt| {
+            QuickJsRuntimeAdapter::do_with(|q_js_rt| {
                 let res = unsafe { q::JS_NewClass(q_js_rt.runtime, class_id, class_def) };
                 log::trace!("new class res {}", res);
                 // todo res should be 0 for ok
@@ -116,8 +121,14 @@ thread_local! {
 
 const MAX_INSTANCE_NUM: usize = u32::MAX as usize;
 
-fn next_id(q_ctx: &QuickJsContext) -> usize {
-    let mappings = &*q_ctx.proxy_instance_id_mappings.borrow();
+pub(crate) fn init_statics() {
+    PROXY_INSTANCE_CLASS_ID.with(|_rc| {
+        //
+    });
+}
+
+fn next_id(proxy: &Proxy) -> usize {
+    let mappings = &*proxy.proxy_instance_id_mappings.borrow();
     if mappings.len() == MAX_INSTANCE_NUM {
         panic!("too many instances"); // todo report ex
     }
@@ -132,9 +143,9 @@ fn next_id(q_ctx: &QuickJsContext) -> usize {
 /// The Proxy struct can be used to create a class in JavaScript who's methods can be implemented in rust
 /// # Example
 /// ```rust
-/// use quickjs_runtime::esruntimebuilder::EsRuntimeBuilder;
+/// use quickjs_runtime::builder::QuickJsRuntimeBuilder;
 /// use quickjs_runtime::reflection::Proxy;
-/// use quickjs_runtime::quickjscontext::QuickJsContext;
+/// use quickjs_runtime::quickjsrealmadapter::QuickJsRealmAdapter;
 /// use quickjs_runtime::valueref::JSValueRef;
 /// use std::cell::RefCell;
 /// use std::collections::HashMap;
@@ -156,7 +167,7 @@ fn next_id(q_ctx: &QuickJsContext) -> usize {
 /// }
 ///
 /// //create a new EsRuntime
-/// let rt = EsRuntimeBuilder::new().build();
+/// let rt = QuickJsRuntimeBuilder::new().build();
 ///
 /// // install our proxy class as com.hirofa.FunkyClass
 /// rt.exe_rt_task_in_event_loop(|q_js_rt| {
@@ -166,7 +177,7 @@ fn next_id(q_ctx: &QuickJsContext) -> usize {
 ///    .name("FunkyClass")
 ///    // the constructor is called when a script does new com.hirofa.FunkyClass, the reflection utils
 ///    // generate an instance_id which may be used to identify the instance
-///    .constructor(|q_ctx: &QuickJsContext, instance_id: usize, args: Vec<JSValueRef>| {
+///    .constructor(|q_ctx: &QuickJsRealmAdapter, instance_id: usize, args: Vec<JSValueRef>| {
 ///        // we'll assume our script always constructs the Proxy with a single name argument
 ///        let name = primitives::to_string_q(q_ctx, &args[0]).ok().expect("bad constructor! bad!");
 ///        // create a new instance of our struct and store it in a map
@@ -218,7 +229,7 @@ fn next_id(q_ctx: &QuickJsContext) -> usize {
 pub struct Proxy {
     name: Option<String>,
     namespace: Option<Vec<String>>,
-    constructor: Option<Box<ProxyConstructor>>,
+    pub(crate) constructor: Option<Box<ProxyConstructor>>,
     finalizers: Vec<Box<ProxyFinalizer>>,
     methods: HashMap<String, Box<ProxyMethod>>,
     native_methods: HashMap<String, ProxyNativeMethod>,
@@ -228,6 +239,7 @@ pub struct Proxy {
     getters_setters: HashMap<String, (Box<ProxyGetter>, Box<ProxySetter>)>,
     is_event_target: bool,
     is_static_event_target: bool,
+    pub(crate) proxy_instance_id_mappings: RefCell<HashMap<usize, Box<ProxyInstanceInfo>>>,
 }
 
 impl Default for crate::reflection::Proxy {
@@ -237,7 +249,7 @@ impl Default for crate::reflection::Proxy {
 }
 
 /// get a proxy by class_name (namespace.ClassName)
-pub fn get_proxy(q_ctx: &QuickJsContext, class_name: &str) -> Option<Rc<Proxy>> {
+pub fn get_proxy(q_ctx: &QuickJsRealmAdapter, class_name: &str) -> Option<Rc<Proxy>> {
     let registry = &*q_ctx.proxy_registry.borrow();
     registry.get(class_name).cloned()
 }
@@ -258,6 +270,7 @@ impl Proxy {
             getters_setters: Default::default(),
             is_event_target: false,
             is_static_event_target: false,
+            proxy_instance_id_mappings: RefCell::new(Default::default()),
         }
     }
 
@@ -309,7 +322,7 @@ impl Proxy {
     /// if omitted the Proxy class will not be constructable from script
     pub fn constructor<C>(mut self, constructor: C) -> Self
     where
-        C: Fn(&QuickJsContext, usize, Vec<JSValueRef>) -> Result<(), JsError> + 'static,
+        C: Fn(&QuickJsRealmAdapter, usize, Vec<JSValueRef>) -> Result<(), JsError> + 'static,
     {
         self.constructor = Some(Box::new(constructor));
         self
@@ -318,7 +331,7 @@ impl Proxy {
     /// this will be called when an instance of the Proxy class is dropped or garbage collected
     pub fn finalizer<C>(mut self, finalizer: C) -> Self
     where
-        C: Fn(&QuickJsContext, usize) + 'static,
+        C: Fn(&QuickJsRealmAdapter, usize) + 'static,
     {
         self.finalizers.push(Box::new(finalizer));
         self
@@ -326,7 +339,8 @@ impl Proxy {
     /// add a method to the Proxy class, this method will be available as a member of instances of the Proxy class
     pub fn method<M>(mut self, name: &str, method: M) -> Self
     where
-        M: Fn(&QuickJsContext, &usize, Vec<JSValueRef>) -> Result<JSValueRef, JsError> + 'static,
+        M: Fn(&QuickJsRealmAdapter, &usize, Vec<JSValueRef>) -> Result<JSValueRef, JsError>
+            + 'static,
     {
         self.methods.insert(name.to_string(), Box::new(method));
         self
@@ -339,7 +353,7 @@ impl Proxy {
     /// add a static method to the Proxy class, this method will be available as a member of the Proxy class itself
     pub fn static_method<M>(mut self, name: &str, method: M) -> Self
     where
-        M: Fn(&QuickJsContext, Vec<JSValueRef>) -> Result<JSValueRef, JsError> + 'static,
+        M: Fn(&QuickJsRealmAdapter, Vec<JSValueRef>) -> Result<JSValueRef, JsError> + 'static,
     {
         self.static_methods
             .insert(name.to_string(), Box::new(method));
@@ -354,8 +368,8 @@ impl Proxy {
     /// add a static getter and setter to the Proxy class
     pub fn static_getter_setter<G, S>(mut self, name: &str, getter: G, setter: S) -> Self
     where
-        G: Fn(&QuickJsContext) -> Result<JSValueRef, JsError> + 'static,
-        S: Fn(&QuickJsContext, JSValueRef) -> Result<(), JsError> + 'static,
+        G: Fn(&QuickJsRealmAdapter) -> Result<JSValueRef, JsError> + 'static,
+        S: Fn(&QuickJsRealmAdapter, JSValueRef) -> Result<(), JsError> + 'static,
     {
         self.static_getters_setters
             .insert(name.to_string(), (Box::new(getter), Box::new(setter)));
@@ -364,8 +378,8 @@ impl Proxy {
     /// add a getter and setter to the Proxy class, these will be available as a member of an instance of this Proxy class
     pub fn getter_setter<G, S>(mut self, name: &str, getter: G, setter: S) -> Self
     where
-        G: Fn(&QuickJsContext, &usize) -> Result<JSValueRef, JsError> + 'static,
-        S: Fn(&QuickJsContext, &usize, JSValueRef) -> Result<(), JsError> + 'static,
+        G: Fn(&QuickJsRealmAdapter, &usize) -> Result<JSValueRef, JsError> + 'static,
+        S: Fn(&QuickJsRealmAdapter, &usize, JSValueRef) -> Result<(), JsError> + 'static,
     {
         self.getters_setters
             .insert(name.to_string(), (Box::new(getter), Box::new(setter)));
@@ -377,13 +391,14 @@ impl Proxy {
         self
     }
     /// indicate the Proxy class should implement the EventTarget interface, this will result in the addEventListener, removeEventListener and dispatchEvent methods to be available
-    pub fn static_event_target(mut self) {
-        self.is_static_event_target = true
+    pub fn static_event_target(mut self) -> Self {
+        self.is_static_event_target = true;
+        self
     }
     /// install the Proxy class in a QuickJsContext, this is always needed as a final step to actually make the Proxy class work
     pub fn install(
         mut self,
-        q_ctx: &QuickJsContext,
+        q_ctx: &QuickJsRealmAdapter,
         add_variable_to_global: bool,
     ) -> Result<JSValueRef, JsError> {
         if self.name.is_none() {
@@ -410,7 +425,7 @@ impl Proxy {
         Ok(class_ref)
     }
 
-    fn install_move_to_registry(self, q_ctx: &QuickJsContext) {
+    fn install_move_to_registry(self, q_ctx: &QuickJsRealmAdapter) {
         let proxy = self;
 
         let reg_map = &mut *q_ctx.proxy_registry.borrow_mut();
@@ -418,7 +433,7 @@ impl Proxy {
     }
     fn install_class_prop(
         &self,
-        q_ctx: &QuickJsContext,
+        q_ctx: &QuickJsRealmAdapter,
         add_variable_to_global: bool,
     ) -> Result<JSValueRef, JsError> {
         // this creates a constructor function, adds it to the global scope and then makes an instance of the static_proxy_class its prototype so we can add static_getters_setters and static_methods
@@ -457,7 +472,7 @@ impl Proxy {
         log::trace!("reflection::Proxy::install_class_prop / 5");
 
         if class_val_ref.is_exception() {
-            return if let Some(e) = unsafe { QuickJsContext::get_exception(q_ctx.context) } {
+            return if let Some(e) = unsafe { QuickJsRealmAdapter::get_exception(q_ctx.context) } {
                 Err(e)
             } else {
                 Err(JsError::new_string(format!(
@@ -476,7 +491,7 @@ impl Proxy {
                 *class_val_ref.borrow_value(),
             );
             if res < 0 {
-                return if let Some(err) = QuickJsContext::get_exception(q_ctx.context) {
+                return if let Some(err) = QuickJsRealmAdapter::get_exception(q_ctx.context) {
                     Err(err)
                 } else {
                     Err(JsError::new_str("could not set class proto"))
@@ -529,16 +544,16 @@ impl Proxy {
 
 pub fn new_instance2(
     proxy: &Proxy,
-    q_ctx: &QuickJsContext,
+    q_ctx: &QuickJsRealmAdapter,
 ) -> Result<(usize, JSValueRef), JsError> {
-    let instance_id = next_id(q_ctx);
+    let instance_id = next_id(proxy);
     Ok((instance_id, new_instance3(proxy, instance_id, q_ctx)?))
 }
 
 pub(crate) fn new_instance3(
     proxy: &Proxy,
     instance_id: usize,
-    q_ctx: &QuickJsContext,
+    q_ctx: &QuickJsRealmAdapter,
 ) -> Result<JSValueRef, JsError> {
     let ctx = q_ctx.context;
     let class_id = PROXY_INSTANCE_CLASS_ID.with(|rc| *rc.borrow());
@@ -569,7 +584,7 @@ pub(crate) fn new_instance3(
         };
     }
 
-    let mappings = &mut *q_ctx.proxy_instance_id_mappings.borrow_mut();
+    let mappings = &mut *proxy.proxy_instance_id_mappings.borrow_mut();
     assert!(!mappings.contains_key(&instance_id));
 
     let mut bx = Box::new(ProxyInstanceInfo {
@@ -589,7 +604,7 @@ pub(crate) fn new_instance3(
 
 pub fn new_instance(
     class_name: &str,
-    q_ctx: &QuickJsContext,
+    q_ctx: &QuickJsRealmAdapter,
 ) -> Result<(usize, JSValueRef), JsError> {
     // todo
 
@@ -622,7 +637,7 @@ unsafe extern "C" fn constructor(
         false,
         "reflection::constructor this_val",
     );
-    QuickJsRuntime::do_with(|q_js_rt| {
+    QuickJsRuntimeAdapter::do_with(|q_js_rt| {
         let name_ref = objects::get_property(context, &this_ref, "name")
             .ok()
             .expect("name get failed");
@@ -638,7 +653,7 @@ unsafe extern "C" fn constructor(
                 // construct
 
                 let args_vec = parse_args(context, argc, argv);
-                let instance_id = next_id(q_ctx);
+                let instance_id = next_id(proxy);
                 let constructor_res = constructor(q_ctx, instance_id, args_vec);
 
                 match constructor_res {
@@ -690,7 +705,7 @@ unsafe extern "C" fn finalizer(_rt: *mut q::JSRuntime, val: q::JSValue) {
     let info: &ProxyInstanceInfo = get_proxy_instance_info(&val);
     trace!("finalize {}", info.id);
 
-    QuickJsRuntime::do_with(|q_js_rt| {
+    QuickJsRuntimeAdapter::do_with(|q_js_rt| {
         let q_ctx = q_js_rt.get_context(&info.context_id);
         log::trace!("finalizer called, got q_ctx");
         let registry = &*q_ctx.proxy_registry.borrow();
@@ -704,7 +719,7 @@ unsafe extern "C" fn finalizer(_rt: *mut q::JSRuntime, val: q::JSValue) {
 
         {
             log::trace!("reflection::finalizer: remove from INSTANCE_ID_MAPPINGS");
-            let id_map = &mut *q_ctx.proxy_instance_id_mappings.borrow_mut();
+            let id_map = &mut *proxy.proxy_instance_id_mappings.borrow_mut();
             let _ = id_map.remove(&info.id).expect("no such id to finalize");
             log::trace!("reflection::finalizer: remove from INSTANCE_ID_MAPPINGS -> done");
         }
@@ -739,7 +754,7 @@ unsafe extern "C" fn proxy_static_get_prop(
         "reflection::proxy_static_get_prop receiver",
     );
 
-    QuickJsRuntime::do_with(|q_js_rt| {
+    QuickJsRuntimeAdapter::do_with(|q_js_rt| {
         let q_ctx = q_js_rt.get_quickjs_context(context);
 
         let proxy_name_ref = objects::get_property(context, &receiver_ref, "name")
@@ -841,7 +856,7 @@ unsafe extern "C" fn proxy_instance_get_prop(
         "reflection::proxy_instance_get_prop receiver",
     );
 
-    QuickJsRuntime::do_with(|q_js_rt| {
+    QuickJsRuntimeAdapter::do_with(|q_js_rt| {
         let q_ctx = q_js_rt.get_quickjs_context(context);
 
         let prop_name = atoms::to_string2(context, &atom)
@@ -899,8 +914,17 @@ unsafe extern "C" fn proxy_instance_get_prop(
             match res {
                 Ok(g_val) => g_val.clone_value_incr_rc(),
                 Err(e) => {
-                    let err = format!("proxy_instance_get_prop failed: {}", e);
-                    q_ctx.report_ex(err.as_str())
+                    let msg = format!("proxy_instance_get failed: {}", e.get_message());
+                    let nat_stack = format!(
+                        "    at Proxy instance getter [{}]\n{}",
+                        prop_name,
+                        e.get_stack()
+                    );
+                    let err =
+                        errors::new_error(context, e.get_name(), msg.as_str(), nat_stack.as_str())
+                            .ok()
+                            .expect("create error failed");
+                    errors::throw(context, err)
                 }
             }
         } else {
@@ -920,8 +944,7 @@ unsafe extern "C" fn proxy_instance_has_prop(
     _obj: q::JSValue,
     _atom: q::JSAtom,
 ) -> ::std::os::raw::c_int {
-    trace!("proxy_instance_has_prop");
-    0
+    todo!()
 }
 #[allow(dead_code)]
 unsafe extern "C" fn proxy_static_has_prop(
@@ -929,8 +952,7 @@ unsafe extern "C" fn proxy_static_has_prop(
     _obj: q::JSValue,
     _atom: q::JSAtom,
 ) -> ::std::os::raw::c_int {
-    trace!("proxy_static_has_prop");
-    0
+    todo!()
 }
 
 unsafe extern "C" fn proxy_instance_method(
@@ -942,7 +964,7 @@ unsafe extern "C" fn proxy_instance_method(
     func_data: *mut q::JSValue,
 ) -> q::JSValue {
     trace!("proxy_instance_method");
-    QuickJsRuntime::do_with(|q_js_rt| {
+    QuickJsRuntimeAdapter::do_with(|q_js_rt| {
         let q_ctx = q_js_rt.get_quickjs_context(context);
 
         let proxy_instance_info: &ProxyInstanceInfo = get_proxy_instance_info(&this_val);
@@ -975,9 +997,15 @@ unsafe extern "C" fn proxy_instance_method(
                 Ok(m_res_ref) => m_res_ref.clone_value_incr_rc(),
                 Err(e) => {
                     let msg = format!("proxy_instance_method failed: {}", e.get_message());
-                    let err = errors::new_error(context, e.get_name(), msg.as_str(), e.get_stack())
-                        .ok()
-                        .expect("create error failed");
+                    let nat_stack = format!(
+                        "    at Proxy instance method [{}]\n{}",
+                        func_name,
+                        e.get_stack()
+                    );
+                    let err =
+                        errors::new_error(context, e.get_name(), msg.as_str(), nat_stack.as_str())
+                            .ok()
+                            .expect("create error failed");
                     errors::throw(context, err)
                 }
             }
@@ -998,7 +1026,7 @@ unsafe extern "C" fn proxy_static_method(
     func_data: *mut q::JSValue,
 ) -> q::JSValue {
     trace!("proxy_static_method");
-    QuickJsRuntime::do_with(|q_js_rt| {
+    QuickJsRuntimeAdapter::do_with(|q_js_rt| {
         let q_ctx = q_js_rt.get_quickjs_context(context);
         let this_ref = JSValueRef::new(
             context,
@@ -1037,8 +1065,17 @@ unsafe extern "C" fn proxy_static_method(
             match m_res {
                 Ok(m_res_ref) => m_res_ref.clone_value_incr_rc(),
                 Err(e) => {
-                    let err = format!("proxy_static_method failed: {}", e);
-                    q_ctx.report_ex(err.as_str())
+                    let msg = format!("proxy_static_method failed: {}", e.get_message());
+                    let nat_stack = format!(
+                        "    at Proxy static method [{}]\n{}",
+                        func_name,
+                        e.get_stack()
+                    );
+                    let err =
+                        errors::new_error(context, e.get_name(), msg.as_str(), nat_stack.as_str())
+                            .ok()
+                            .expect("create error failed");
+                    errors::throw(context, err)
                 }
             }
         } else {
@@ -1049,32 +1086,138 @@ unsafe extern "C" fn proxy_static_method(
 }
 
 unsafe extern "C" fn proxy_static_set_prop(
-    _context: *mut q::JSContext,
+    context: *mut q::JSContext,
     _obj: q::JSValue,
-    _atom: q::JSAtom,
-    _value: q::JSValue,
-    _receiver: q::JSValue,
+    atom: q::JSAtom,
+    value: q::JSValue,
+    receiver: q::JSValue,
     _flags: ::std::os::raw::c_int,
 ) -> ::std::os::raw::c_int {
-    trace!("proxy_static_set_prop");
-    0
+    trace!("proxy_instance_set_prop");
+
+    let value_ref = JSValueRef::new(
+        context,
+        value,
+        false,
+        false,
+        "reflection::proxy_instance_set_prop value",
+    );
+    let receiver_ref = JSValueRef::new(
+        context,
+        receiver,
+        false,
+        false,
+        "reflection::proxy_instance_set_prop value",
+    );
+
+    QuickJsRuntimeAdapter::do_with(|q_js_rt| {
+        let q_ctx = q_js_rt.get_quickjs_context(context);
+
+        let prop_name = atoms::to_string2(context, &atom)
+            .ok()
+            .expect("could not get name");
+        trace!("proxy_static_set_prop: {}", prop_name);
+
+        // see if we have a matching gettersetter
+
+        let proxy_name_ref = objects::get_property(context, &receiver_ref, "name")
+            .ok()
+            .unwrap();
+        let proxy_name = primitives::to_string(context, &proxy_name_ref)
+            .ok()
+            .unwrap();
+        trace!("proxy_static_get_prop: {}", proxy_name);
+
+        let prop_name = atoms::to_string2(context, &atom)
+            .ok()
+            .expect("could not get name");
+        trace!("proxy_static_get_prop: prop: {}", prop_name);
+
+        let registry = &*q_ctx.proxy_registry.borrow();
+        if let Some(proxy) = registry.get(proxy_name.as_str()) {
+            if let Some(getter_setter) = proxy.static_getters_setters.get(&prop_name) {
+                // call the setter
+                let setter = &getter_setter.1;
+                let res: Result<(), JsError> = setter(q_ctx, value_ref);
+                match res {
+                    Ok(_) => 0,
+                    Err(e) => {
+                        // fail, todo do i need ex?
+                        let err = format!("proxy_instance_set_prop failed: {}", e);
+                        log::error!("{}", err);
+                        //let _ = q_ctx.report_ex(err.as_str());
+                        -1
+                    }
+                }
+            } else {
+                // fail
+                -1
+            }
+        } else {
+            -1
+        }
+    })
 }
 
 unsafe extern "C" fn proxy_instance_set_prop(
-    _context: *mut q::JSContext,
-    _obj: q::JSValue,
-    _atom: q::JSAtom,
-    _value: q::JSValue,
+    context: *mut q::JSContext,
+    obj: q::JSValue,
+    atom: q::JSAtom,
+    value: q::JSValue,
     _receiver: q::JSValue,
     _flags: ::std::os::raw::c_int,
 ) -> ::std::os::raw::c_int {
     trace!("proxy_instance_set_prop");
-    0
+
+    let value_ref = JSValueRef::new(
+        context,
+        value,
+        false,
+        false,
+        "reflection::proxy_instance_set_prop value",
+    );
+
+    QuickJsRuntimeAdapter::do_with(|q_js_rt| {
+        let q_ctx = q_js_rt.get_quickjs_context(context);
+
+        let prop_name = atoms::to_string2(context, &atom)
+            .ok()
+            .expect("could not get name");
+        trace!("proxy_instance_set_prop: {}", prop_name);
+
+        let info = get_proxy_instance_info(&obj);
+
+        trace!("obj_ref.classname = {}", info.class_name);
+
+        // see if we have a matching gettersetter
+
+        let registry = &*q_ctx.proxy_registry.borrow();
+        let proxy = registry.get(&info.class_name).unwrap();
+
+        if let Some(getter_setter) = proxy.getters_setters.get(&prop_name) {
+            // call the setter
+            let setter = &getter_setter.1;
+            let res: Result<(), JsError> = setter(q_ctx, &info.id, value_ref);
+            match res {
+                Ok(_) => 0,
+                Err(e) => {
+                    // fail, todo do i need ex?
+                    let err = format!("proxy_instance_set_prop failed: {}", e);
+                    log::error!("{}", err);
+                    //let _ = q_ctx.report_ex(err.as_str());
+                    -1
+                }
+            }
+        } else {
+            // fail
+            -1
+        }
+    })
 }
 
 #[cfg(test)]
 pub mod tests {
-    use crate::esruntime::tests::init_test_rt;
+    use crate::facades::tests::init_test_rt;
     use crate::quickjs_utils::{functions, primitives};
     use crate::reflection::Proxy;
     use hirofa_utils::js_utils::JsError;
@@ -1107,6 +1250,34 @@ pub mod tests {
         });
     }
 
+    #[test]
+    pub fn test_proxy_ex() {
+        log::info!("> test_proxy");
+
+        let rt = init_test_rt();
+        let err = rt.exe_rt_task_in_event_loop(|q_js_rt| {
+            q_js_rt.gc();
+            let q_ctx = q_js_rt.get_main_context();
+            let _ = Proxy::new()
+                .constructor(|_q_ctx, _id, _args| Ok(()))
+                .method("run", |_realm, _instance_id, _args| {
+                    Err(JsError::new_str("cant run"))
+                })
+                .name("Test")
+                .install(q_ctx, true);
+            let err = q_ctx
+                .eval(Script::new("test.es", "let t = new Test(); \nt.run();"))
+                .err()
+                .expect("script failed");
+
+            format!("{}", err)
+        });
+
+        assert!(err.contains("test.es:2"));
+        assert!(err.contains("at Proxy instance method [run]"));
+        assert!(err.contains("cant run"));
+    }
+
     #[test]
     pub fn test_to_string() {
         log::info!("> test_proxy");
@@ -1261,6 +1432,8 @@ pub mod tests {
         let e_msg = e.get_message();
         assert_eq!(e_msg, "proxy_instance_method failed: aaargh");
 
+        assert!(e.get_stack().contains("[doIt2]"));
+
         rt.gc_sync();
 
         std::thread::sleep(Duration::from_secs(1));
